/*
 * error_assert_handler.c
 *
 *  Created on: Oct 8, 2015
 *      Author: Prithvi
 */

#include <stdint.h>
#include <tfp_printf.h>

/**@brief Function for error handling, which is called when an error has occurred.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of error.
 *
 * @param[in] error_code  Error code supplied to the handler.
 * @param[in] line_num    Line number where the handler is called.
 * @param[in] p_file_name Pointer to the file name.
 */
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name){
  // This call can be used for ONLY debug purposes during application development.
  tfp_printf("Error of 0x%X at %d in  %s\n", error_code, line_num, p_file_name);

  for(;;);

  // On assert, the system can only recover with a reset.
  //NVIC_SystemReset();
}


void assert_softdevice_callback(uint16_t line_num, const uint8_t *file_name){
  // This call can be used for ONLY debug purposes during application development.
  tfp_printf("SD assert at %d in  %s\n", line_num, file_name);

  for(;;);

  // On assert, the system can only recover with a reset.
  //NVIC_SystemReset();
}


/**
 * @brief The assert handler that prints the file and line where the error is
 * @param[in] line_num    Line number where the assert handler is called.
 * @param[in] file_name Pointer to the file name.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t *file_name){
  // This call can be used for ONLY debug purposes during application development.
  tfp_printf("App assert at %d in  %s\n", line_num, file_name);

  for(;;);
}
