/*
 * radio_notif.c
 *
 * Author: Arjun
 */
#include "radio_notif.h"
#include "softdevice_handler.h"
#include "timeslot.h"
#include "tfp_printf.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"

#define TIMESLOT_US   400

#define USE_TIMESLOT 1

#if USE_TIMESLOT==1

static bool enable_notif = false;

static struct {
  uint8_t irq_priority;
  uint8_t type;
  uint8_t distance;
}radio_notif_param;

void SWI0_IRQHandler(void){

  uint32_t err_code = sd_nvic_ClearPendingIRQ(SWI0_IRQn);
  APP_ERROR_CHECK(err_code);
  err_code = sd_nvic_DisableIRQ(SWI0_IRQn);
  APP_ERROR_CHECK(err_code);

  //tfp_printf("B%d\n", enable_notif);

  if(enable_notif){
    err_code = sd_nvic_ClearPendingIRQ(SWI1_IRQn);
    APP_ERROR_CHECK(err_code);
    err_code = sd_nvic_EnableIRQ(SWI1_IRQn);
    APP_ERROR_CHECK(err_code);

    // Configure the event
    err_code = sd_radio_notification_cfg_set(radio_notif_param.type,
        radio_notif_param.distance);
    APP_ERROR_CHECK(err_code);
  } else {
    err_code = sd_nvic_ClearPendingIRQ(SWI1_IRQn);
    APP_ERROR_CHECK(err_code);
    err_code = sd_nvic_DisableIRQ(SWI1_IRQn);
    APP_ERROR_CHECK(err_code);
    err_code = sd_radio_notification_cfg_set(
          NRF_RADIO_NOTIFICATION_TYPE_NONE, NRF_RADIO_NOTIFICATION_DISTANCE_NONE);
    APP_ERROR_CHECK(err_code);
  }

  timeslot_close();

}

nrf_radio_signal_callback_return_param_t * timeslot_callback(uint8_t signal_type){

  static nrf_radio_signal_callback_return_param_t signal_callback_return_param;

  signal_callback_return_param.params.request.p_next = NULL;
  signal_callback_return_param.callback_action = NRF_RADIO_SIGNAL_CALLBACK_ACTION_NONE;

  //tfp_printf("A%d\n", signal_type);

  switch(signal_type){
    case NRF_RADIO_CALLBACK_SIGNAL_TYPE_START:

      NVIC_EnableIRQ(SWI0_IRQn);
      NVIC_SetPendingIRQ(SWI0_IRQn);

      break;
    default:
      break;

  }

  return &signal_callback_return_param;
}
#endif

void radio_notif_init(uint32_t irq_priority, uint32_t notif_type, uint32_t distance){
#if USE_TIMESLOT==1
  radio_notif_param.irq_priority = irq_priority;
  radio_notif_param.type = notif_type;
  radio_notif_param.distance = distance;
#endif

  uint32_t err_code;

  // Initialize Radio Notification software interrupt
  err_code = sd_nvic_ClearPendingIRQ(SWI1_IRQn);
  APP_ERROR_CHECK(err_code);

  err_code = sd_nvic_SetPriority(SWI1_IRQn, irq_priority);
  APP_ERROR_CHECK(err_code);

  err_code = sd_nvic_DisableIRQ(SWI1_IRQn);
  APP_ERROR_CHECK(err_code);

#if USE_TIMESLOT==1
  // Initialize IRQ handler to enable radio notification
  err_code = sd_nvic_ClearPendingIRQ(SWI0_IRQn);
  APP_ERROR_CHECK(err_code);

  err_code = sd_nvic_SetPriority(SWI0_IRQn, APP_IRQ_PRIORITY_LOW);
  APP_ERROR_CHECK(err_code);

  err_code = sd_nvic_DisableIRQ(SWI0_IRQn);
  APP_ERROR_CHECK(err_code);
#else
  // Configure the event
  err_code = sd_radio_notification_cfg_set(notif_type, distance);
  APP_ERROR_CHECK(err_code);
#endif
}

void radio_notif_enable(void){
#if USE_TIMESLOT==1
  enable_notif = true;
  timeslot_sd_open_start_earliest(TIMESLOT_US, timeslot_callback);
#else
  uint32_t err_code = sd_nvic_EnableIRQ(SWI1_IRQn);
  APP_ERROR_CHECK(err_code);

  // Configure the event
  err_code = sd_radio_notification_cfg_set(radio_notif_param.type,
      radio_notif_param.distance);
  APP_ERROR_CHECK(err_code);
#endif
}


void radio_notif_disable(void){
#if USE_TIMESLOT==1
  enable_notif = false;
  timeslot_sd_open_start_earliest(TIMESLOT_US, timeslot_callback);
#else
  uint32_t err_code = sd_radio_notification_cfg_set(
        NRF_RADIO_NOTIFICATION_TYPE_NONE, NRF_RADIO_NOTIFICATION_DISTANCE_NONE);
    APP_ERROR_CHECK(err_code);

  uint32_t err_code = sd_nvic_DisableIRQ(SWI1_IRQn);
  APP_ERROR_CHECK(err_code);
#endif
}
