/*
 * radio_notif.h
 *
 *  Created on: Oct 9, 2015
 *      Author: basslet
 */

#ifndef CODEBASE_PERIPHERALS_RADIO_NOTIF_H_
#define CODEBASE_PERIPHERALS_RADIO_NOTIF_H_

/** @file
 *
 * @defgroup radio_notification Radio Notification Event Handler
 * @{
 * @brief Module for propagating Radio Notification events to the application.
 */

#include <stdint.h>
#include <stdbool.h>
#include "nrf_soc.h"
#include "app_error.h"

#define radio_notif_handler     SWI1_IRQHandler

/**@brief Function for initializing the Radio Notification module.
 *
 * @param[in]  irq_priority   Interrupt priority for the Radio Notification interrupt handler.
 * @param[in]  notif_type     Mention the mode of notif, if the generated before, after or
 *                            both before and after radio event @NRF_RADIO_NOTIFICATION_TYPES
 * @param[in]  distance       The time from an Active event until the radio is activated.
 */
void radio_notif_init(uint32_t irq_priority, uint32_t notif_type, uint32_t distance);

/**
 * @brief Enable the SWI1 IRq for radio notification interrupt
 */
void radio_notif_enable(void);

/**
 * @brief Enable the SWI1 IRq for radio notification interrupt
 */
void radio_notif_disable(void);

/** @} */

#endif /* CODEBASE_PERIPHERALS_RADIO_NOTIF_H_ */
